@@ -2,35 +2,40 @@
 # -*-coding:utf-8 -*-
 
 #********************************************************************************
-#Author: tony - birdaccp@gmail.com alex8224@gmail.com
+#Author: alex8224@gmail.com
 #Create by: 2013-08-17 12:53
 #Last modified: 2014-07-01
 #Filename: kuanpanfuse.py
-#Description: 实现快盘的ＡＰＩ,可以实现文件的异步上传, 下载，浏览，目录操作
+#Description: 
+# 实现快盘的ＡＰＩ,可以实现文件的异步上传, 下载，浏览，目录, 文档转换操作
+# 支持配置本地读缓存，支持异步写
+# 支持查看网盘容量
+# 支持文档转换，以虚拟打印机的方式实现
 # 缩略图和版本功能未实现
 # todo
-# 1. bulid entire director tree
-# from root get file list. 判断如果是目录，则继续调用ＡＰＩ，轮询该目录,如果该目录下没有其他目录，则返回上一级
-# 使用stack代替递归，how?
-# 2. make wrietask and download task managmented by threadpool
-# 3. add virtualdirectory support extend task, for example: document convert
+# README.MD on github
+# 3. test host speed and using fastest connection download file
 #********************************************************************************
+
 import os
 import sys
 import time
 import copy
 import signal
-import common
+import pycurl
+from common import *
 from Queue import Queue
 from hashlib import sha1
 from functools import partial
 from collections import deque
 from stat import S_IFDIR, S_IFREG
-from kuaipanapi import OpenAPIError, OpenAPIException
-from threading import Thread, Condition, RLock as Lock
+from types import FunctionType, MethodType
 from errno import ENOENT, EROFS, EEXIST, EIO
+from threading import Thread, Condition, RLock as Lock
+from kuaipanapi import KuaipanAPI, OpenAPIError, OpenAPIException
 from fuse import FUSE, FuseOSError, Operations, LoggingMixIn, fuse_get_context
 
+logger = None
 TYPE_DIR = (S_IFDIR | 0644 )
 TYPE_FILE = (S_IFREG | 0644 )
 BLOCK_SIZE = 4096
@@ -38,33 +43,19 @@ BLOCK_SIZE = 4096
 CACHE_PATH = "/dev/shm/"
 
 ROOT_ST_INFO = {
-        "st_mtime": common.timestamp(),
+        "st_mtime": timestamp(),
         "st_mode":  TYPE_DIR,
-        "st_size":  4096,
+        "st_size":  BLOCK_SIZE,
         "st_gid":   os.getgid(),
         "st_uid":   os.getuid(),
-        "st_atime": common.timestamp(),
+        "st_atime": timestamp(),
         "st_nlink": 2,
         "st_parent_dir":".",
         "st_name": "/"
         }
 
-import logging
-def getlogger():
-    logger = logging.getLogger("kuaipanserver")
-    hdlr = logging.StreamHandler()
-    fdlr = logging.handlers.TimedRotatingFileHandler("kuaipan_fuse.log", 'D', backupCount=30)
-    format = logging.Formatter("%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s - %(message)s", "%Y-%m-%d %H:%M:%S")
-    logger.addHandler(hdlr)
-    logger.addHandler(fdlr)
-    hdlr.setFormatter(format)
-    fdlr.setFormatter(format)
-    logger.setLevel(logging.DEBUG)
-    return logger
 
 
-logger = getlogger()
-
 def handler(signum, frame):
     print "use press ctrl+c exit"
     sys.exit(0)
@@ -101,18 +92,21 @@ class Future(object):
     def is_finished(self):
         return self.finished
 
+class Callable(Future):
+    def __init__(self, callable_):
+        super(Callable, self).__init__()
+        self.callable_ = callable_
 
-class ThreadPool(Thread):
+    def __call__(self, *args, **kwargs):
+        return self.callable_(*args, **kwargs)
 
-    _instance_lock = Lock()
+class ThreadPool(Singleton, Thread):
 
     def __init__(self):
         Thread.__init__(self)
-        from config import Config
-        configobj = Config()
-        self.max_thread_num = int(configobj.get_max_thread_num())
-        self.free_thread_num = int(configobj.get_free_thread_num())
-        self.idle_seconds = int(configobj.get_idle_seconds())
+        self.max_thread_num = int(config.get_max_thread_num())
+        self.free_thread_num = int(config.get_free_thread_num())
+        self.idle_seconds = int(config.get_idle_seconds())
         self.runflag = False
         self.freequeue = deque()
         self.busyqueue = deque()
@@ -122,21 +116,13 @@ class ThreadPool(Thread):
         self.setDaemon(True)
         self.initpool()
 
-    @staticmethod
-    def instance():
-        '''create singleton ThreadPool object'''
-        if not hasattr(ThreadPool, "_instance"):
-            with ThreadPool._instance_lock:
-                if not hasattr(ThreadPool, "_instance"):
-                    ThreadPool._instance = ThreadPool()
-        return ThreadPool._instance
-
 
     def initpool(self):
         for workerindex in range(self.free_thread_num):
             self.__createworker()
 
     def quit(self):
+
         '''wait for all busy task done'''''
         while 1:
             with self.condition:
@@ -160,8 +146,14 @@ class ThreadPool(Thread):
     def __dotaskinworker(self, callable_, *args, **kwargs):
         worker = self.freequeue.popleft()
         self.busyqueue.append(worker)
+        if type(callable_) in (FunctionType, MethodType):
+            callable_ = Callable(callable_)
+
         worker.execute(callable_, *args, **kwargs)
-        return self.future(callable_, worker)
+        if callable(callable_):
+            return callable_
+        else:
+            return worker
 
     def delay(self, callable_, timeout):
         '''schedule a delay task'''
@@ -177,17 +169,8 @@ class ThreadPool(Thread):
             _peroidicwrapper.callablefunc = callable_
             self.delayqueue.append((time.time(), interval, _peroidicwrapper))
 
-    def future(self, callable_, worker):
-        '''get future result'''
-        if issubclass(callable_.__class__, Future):
-            return callable_
-        else:
-            return worker
-
     def submit(self, callable_, *args, **kwargs):
-
         with self.lock:
-
             idle_threadsize = len(self.freequeue)
             busy_threadsize = len(self.busyqueue)
             all_threadsize = idle_threadsize + busy_threadsize
@@ -200,14 +183,12 @@ class ThreadPool(Thread):
                 return self.__dotaskinworker(callable_, *args, **kwargs)
 
             elif idle_threadsize == 0 and all_threadsize == self.max_thread_num:
-                logger.debug("no available thread, wait for notify...")
                 self.condition.wait()
                 return self.__dotaskinworker(callable_, *args, **kwargs)
 
     def taskdone(self, worker):
         with self.condition:
             self.condition.notify()
-            logger.debug("worker:(%s) task done" % str(worker))
             self.freequeue.append(worker)
             self.busyqueue.remove(worker)
             
@@ -276,7 +257,6 @@ class Worker(Thread, Future):
         while 1:
             methodname, args = self.interactivequeue.get()
             if methodname == "end_task":
-                logger.debug("interactive terminated")
                 self.status = "FREE"
                 self.lastupdate = time.time()
                 ThreadPool.instance().taskdone(self)
@@ -287,6 +267,7 @@ class Worker(Thread, Future):
             else:
                 logger.error("no such method %s" % methodname)
 
+
     def runascallable(self, task, *args, **kwargs):
         try:
            self.status = "RUNNING"
@@ -295,13 +276,13 @@ class Worker(Thread, Future):
            self.status = "FREE"
            self.lastupdate = time.time()
         except Exception, taskex:
-           task.set_result(result)
-           self.status = "FREE"
-           logger.error(taskex)
+            print "-=============", task
+            logger.error(taskex)
+            task.set_result(taskex)
+            self.status = "FREE"
         finally:
            ThreadPool.instance().taskdone(self)
 
-        pass
 
     def run(self):
         while 1:
@@ -324,17 +305,13 @@ class TaskPool(object):
         self.uploadlock, self.downlock = Lock(), Lock()
         self.taskclass = {"upload":WriteTask, "download":DownloadTask}
 
-    def upload_file(self, hashpath, *args):
+    def upload_file(self, hashpath, *args, **kwargs):
         params = list(args)
         params.insert(1, hashpath)
-        task = ThreadPool.instance().submit(WriteTask(*params))
+        task = ThreadPool.instance().submit(WriteTask(*params, **kwargs))
         self.uploadpool[hashpath] = task
         return task
 
-    def query_upload_task1(self, key):
-        if key in self.uploadpool:
-            return self.uploadpool[key]
-
     def delete_upload_task(self, key):
         if key in self.uploadpool:
             del self.uploadpool[key]
@@ -397,9 +374,10 @@ class WalkAroundTreeTask(Future):
         Future.__init__(self)
         self.api = api
         self.path = path
+        self.session = HTTPSession()
 
     def readdir(self, path):
-        return self.api.metadata(path=path)
+        return self.api.metadata(path=path, session=self.session)
 
     def listdir(self, path, allfilesinfo):
         dirs = []
@@ -412,12 +390,12 @@ class WalkAroundTreeTask(Future):
             fullpath = os.path.join(path, finfo["name"])
 
             st_info = {
-                        "st_mtime": common.to_timestamp(finfo["modify_time"]),
+                        "st_mtime": to_timestamp(finfo["modify_time"]),
                         "st_mode":  TYPE_FILE if finfo["type"] == "file" else TYPE_DIR,
-                        "st_size":  int(finfo["size"]) if finfo["type"] == "file" else 4096,
+                        "st_size":  int(finfo["size"]) if finfo["type"] == "file" else BLOCK_SIZE,
                         "st_gid":   os.getgid(),
                         "st_uid":   os.getuid(),
-                        "st_atime": common.timestamp(),
+                        "st_atime": timestamp(),
                         "st_parent_dir": path,
                         "st_name": finfo["name"]
                     }
@@ -449,7 +427,9 @@ class WalkAroundTreeTask(Future):
                     if nodetype == "dir":
                         stack.append(path)
             except:
-                pass
+                import traceback;traceback.print_exc()
+
+        self.session.close()            
         return allfilesinfo
 
     def __call__(self, *args):
@@ -488,6 +468,8 @@ class FuseTask(Future):
         return self.api.delete(path)
 
     def unlink(self, path):
+        hashpath = sha1(path.encode("utf-8")).hexdigest().strip()
+        SafeLRUCache.instance().remove(hashpath)
         return self.rmdir(path)
 
     def rename(self, oldpath, newpath):
@@ -497,152 +479,122 @@ class FuseTask(Future):
         return self.api.account_info()
 
 
-# class DownloadTask(Thread):
-    # def __init__(self, api, hashpath, path, bufsize, notify):
-        # Thread.__init__(self)
-        # self.api = api
-        # self.path = path
-        # self.hashpath = hashpath
-        # self.condi = Condition()
-        # self.downloadbytes = 0
-        # self.finished = False
-        # import tempfile
-        # self.tmpfilename = tempfile.mktemp()
-        # self.continueread = True
-        # self.setDaemon(True)
+class DownloadTask(Thread):
 
-    # def wait_data(self, size, offset):
-        # while self.continueread:
-            # with self.condi:
-                # if offset < self.downloadbytes:
-                    # with open(self.tmpfilename) as ronlyfile:
-                        # ronlyfile.seek(offset)
-                        # return ronlyfile.read(size)
-                # elif self.finished:
-                    # break
-                # else:
-                    # self.condi.wait(5)
-                    # logger.debug("%d:%d:%d:%s" % (offset, size, self.downloadbytes, self.tmpfilename))
-
-    # def end_download_file(self):
-        # with self.condi:
-            # self.continueread = False
-            # self.condi.notify_all()
-            # logger.debug("clean cache %s" % self.tmpfilename)
-            # if os.path.exists(self.tmpfilename):
-                # os.unlink(self.tmpfilename)
-
-    # def run(self):
-        # self.stream = self.api.download_file1(self.path)
-        # with open(self.tmpfilename, "w") as tempfile:
-            # while 1:
-                # data = self.stream.read(8192)
-                # if not data:
-                    # break
-                # tempfile.write(data)
-                # tempfile.flush()
-                # with self.condi:
-                    # self.downloadbytes += len(data)
-                    # self.condi.notify_all()
-        # self.stream.close()
-        # with self.condi: 
-            # self.finished = True
-            # self.condi.notify_all()
-        # logger.debug("%s download ok, hashpath:%s" % (self.path, self.hashpath))
-
-# class DownloadTask(Thread, Future):
-
-    # def __init__(self, api, hashpath, path, bufsize, notify):
-        # Thread.__init__(self)
-        # Future.__init__(self)
-        # self.api = api
-        # self.path = path
-        # self.hashpath = hashpath
-        # from threading import Event
-        # self.waiter = Event()
-        # self.isfirst = True
-        # self.bufsize = bufsize
-        # self.notify = notify
-        # self.downloadbytes = 0
-        # self.lock = Lock()
-        # self.waitobj = Condition(self.lock)
-
-    # def wait_data(self, size):
-        # self.waiter.wait()
-        # with self.lock:
-            # if self.isfirst:
-                # self.buffgen.send(None)
-                # firstdata = self.buffgen.send(self.bufsize)
-                # self.downloadbytes += len(firstdata)
-                # self.isfirst = False
-                # return firstdata
-            # else:
-                # try:
-                    # data = self.buffgen.send(size)
-                    # self.downloadbytes += len(data)
-                    # return data
-                # except StopIteration:
-                    # logger.error("gen stoped============================")
-                    # self.buffgen.close()
-                # except Exception, e:
-                    # import traceback
-                    # traceback.print_exc()
-
-    # def end_download_file(self):
-        # logger.debug("file %s download completed, %d bytes downloaded, key %s" % (self.path, self.downloadbytes, self.hashpath))
-        # with self.lock:
-            # self.notify()
-            # self.buffgen.close()
-
-    # def run(self):
-        # logger.debug("start download %s" % self.path)
-        # with self.lock:
-            # self.buffgen = self.api.download_file(self.path, self.bufsize)
-            # self.waiter.set()
-
-class DownloadTask(object):
-
-    def __init__(self, api, hashpath, path, bufsize, notify):
+    def __init__(self, api, hashpath, path, filesize, notify):
+        Thread.__init__(self)
         self.api = api
+        self.hashpath = hashpath
+        self.cachekey = hashpath[0:hashpath.find("_")]
         self.path = path
-        self.stream = None
-        self.lock = Lock()
+        self.filesize = filesize
+        self.condition = Condition()
+        self.cache = SafeLRUCache.instance()
+        self.cachefile = CopyOnWriteBuffer()
+        self.fromcache = False
+        self.cachevalue = self.querycache()
+        self.success = True
 
-    def wait_data(self, size):
-        with self.lock:
-            if self.stream:
-                return self.stream.read(size)
-            else:
-                logger.debug("stream not init. expect size is:%d" % size)
+    def querycache(self):
+        cachefile = self.cache.get(self.cachekey)
+        if cachefile:
+            try:
+                value = cachefile.value
+                self.fromcache = True
+                return value
+            except IOError, ioe:
+                logger.error(ioe)
+                self.cache.remove(self.cachekey)
 
-    def close_stream(self):
-        with self.lock:
-            if self.stream:
-                self.stream.close()
+    def savetocache(self, data):
+        diskcachefile = DiskCacheable()
+        diskcachefile.key = self.cachekey
+        diskcachefile.value = data
+        self.cache.set(self.cachekey, diskcachefile)
+        logger.debug("save %s to cachefile" % self.hashpath)
+
+    def readfromcache(self, offset, size):
+        with self.condition:
+            cachefile = self.cachevalue
+            if cachefile:
+                if offset > self.filesize:
+                    return ''
+                cachefile.seek(offset)
+                return cachefile.read(size)
+
+    def readfromserver(self, offset, size):
+
+        while 1:
+            with self.condition:
+                if not self.success:
+                    return ''
+                cachefilesize = self.cachefile.length
+                if offset + size < cachefilesize:
+                    self.cachefile.seek(offset)
+                    return self.cachefile.read(size)
+                elif cachefilesize == self.filesize:
+                    if offset < cachefilesize:
+                        self.cachefile.seek(offset)
+                        return self.cachefile.read(size)
+                    else:
+                        #why offfset > length?
+                        return ''
+                else:
+                    self.condition.wait(1)
+
+    def wait_data(self, offset, size):
+        if self.fromcache:
+            return self.readfromcache(offset, size)
+        else:
+            return self.readfromserver(offset, size)
 
     def end_download_file(self):
-        with self.lock:
-            logger.debug("file %s download completed" % (self.path, ))
-            self.close_stream()
+        with self.condition:
+            logger.debug("file %s download completed" % self.path)
+            if not self.fromcache and self.success:
+                self.cachefile.seek(0)
+                self.savetocache(self.cachefile.read())
+            self.notify()
 
-    def start(self):
-        with self.lock:
+    def notify(self):
+        with self.condition:
+            self.condition.notify()
+
+    def run(self):
+        if not self.fromcache:
             logger.debug("start download %s" % self.path)
-            self.stream = self.api.download_file1(self.path)
+            session = HTTPSession()
+            try:
+                urlresult = self.api.get_downloadurl(self.path, session)
+                if urlresult.status_code != 302:
+                    raise OpenAPIException("get_download_url failed %s" % urlresult.text)
+                downloadurl = urlresult.headers["location"]
+                self.api.download_file2(downloadurl, self.notify, self.cachefile, session)
+            except (OpenAPIError, OpenAPIException) as openex:
+                logger.debug(openex)
+                self.success = False
+                self.notify()
+            except pycurl.error as pyerror:
+                logger.debug("download file error, reason is %s" % pyerror[1])
+                self.success = False
+                self.notify()
+            finally:
+                session.close()
+        else:
+            logger.debug("download file from cache")
 
 
 class WriteTask(object):
 
-    def __init__(self, api, hashpath, path, filename, uploadqueue):
+    def __init__(self, api, hashpath, path, filename, uploadqueue, handler=None):
         self.api = api
         self.hashpath = hashpath
         self.path = path
         self.filename = filename
         self.fullpath = CACHE_PATH + hashpath
-        self.writebytes = 0
-        self.filesize = 0
         self.uploadqueue = uploadqueue
         self.clsname = "_" + self.__class__.__name__
+        self.afteruploadhandler = handler 
 
 
     def start_upload_file(self, data, offset):
@@ -660,7 +612,6 @@ class WriteTask(object):
                 f.seek(offset)
                 f.write(data)
 
-
     def end_upload_file(self):
         def upload_file():
             try:
@@ -669,6 +620,7 @@ class WriteTask(object):
                 uploadpath = os.path.dirname(self.path)
                 uploadresult = self.api.upload(uploadpath, self.fullpath, self.filename)
                 if uploadresult:
+                    self.afteruploadhandler(self.api)
                     logger.debug("file %s upload ok" % self.filename)
                     return True
                 else:
@@ -699,10 +651,46 @@ class WriteTask(object):
             logger.error(e)
 
 
+class AfterUploadHandler(object):
+    def __init__(self, api, path, fuse):
+        self.api = api
+        self.path = path
+        self.outputdir = config.get_file_convert_out_dir()
+        self.virtualdir = config.get_virtual_convert_dir()
+        self.threadpool = ThreadPool.instance()
+        self.fuse = fuse
+
+    def async_convert(self, api, viewtype):
+        if not os.path.exists(self.outputdir):
+            os.mkdir(self.outputdir)
+
+        fullpath = os.path.join(self.outputdir, os.path.basename(self.path)) + ".zip"
+        result = api.convert(self.path, viewtype)
+        if result.status_code == 200:
+            with open(fullpath, "w") as zipfile:
+                zipfile.write(result.raw.read())
+
+            logger.debug("%s convert ok" % fullpath)
+            self.fuse.unlink(self.path)
+        else:
+            logger.debug("convert document failed, the reason is %s" % result.text)
+
+    def __call__(self, *args):
+        dirname = os.path.dirname(self.path)
+        if dirname != self.virtualdir:
+            return
+        logger.debug("converting %s to html format to %s" % (self.path, self.outputdir))
+        _, extname = os.path.splitext(self.path)
+        viewtype = "normal"
+        if extname in ('.pdf', '.doc', '.wps', '.csv', '.prn', '.xls', '.et', '.ppt', '.dps', '.txt', '.rtf'):
+            self.threadpool.submit(self.async_convert, self.api, viewtype)
+        else:
+            logger.debug("not support format: %s" % extname)
+
 class KuaiPanFuse(LoggingMixIn, Operations):
 
-    def __init__(self, api):
-        self.api = api
+    def __init__(self):
+        self.api = KuaipanAPI.instance()
         self.root = "."
         self.fd = 0
         self.fileprops = {"/":ROOT_ST_INFO}
@@ -710,25 +698,23 @@ class KuaiPanFuse(LoggingMixIn, Operations):
         self.rootfiles = []
         self.cwd = ''
         self.rlock = Lock()
-        self.quota = {"f_blocks":0, "f_bavail":0}
+        self.quota = {"f_blocks":0, "f_bavail":0, "totalspace":0, "usedspace":0}
         self.walked = False
         self.uploadqueue = {}
 
     def initkuaipan(self):
         localtree = self.loadtree()
-        logger.debug(localtree)
 
         if localtree: 
             self.fileprops = localtree
         else:
             logger.warn("no tree.db")
-
+        self.mkdir(config.get_virtual_convert_dir(), 0644)
         ThreadPool.instance().start()
         threadpool = ThreadPool.instance()
         threadpool.delay(self.setaccountinfo, 0)
         threadpool.peroidic(self.setaccountinfo, 300)
         threadpool.delay(self.updatefileprops, 1)
-
         
     def __call__(self, op, *args):
         try:
@@ -738,12 +724,14 @@ class KuaiPanFuse(LoggingMixIn, Operations):
                 raise FuseOSError(EIO)
             else:
                 raise
-
-
+            
     def init(self, path):
         logger.debug("fuse system already started!")
         self.initkuaipan()
+        self.initcache()
 
+    def initcache(self):
+        SafeLRUCache.instance().load()
 
     def setaccountinfo(self):
 
@@ -754,9 +742,8 @@ class KuaiPanFuse(LoggingMixIn, Operations):
                 availspace = totalspace - usedspace
                 self.quota["f_blocks"] = totalspace / BLOCK_SIZE
                 self.quota["f_bavail"] = availspace / BLOCK_SIZE
-                logger.debug("totalspace=%d, usedspace=%d" % (totalspace, usedspace))
             except:
-                pass
+               import traceback;traceback.print_exc()
 
         future = ThreadPool.instance().submit(FuseTask("accountinfo", self.api))
         result = future.get()
@@ -801,12 +788,15 @@ class KuaiPanFuse(LoggingMixIn, Operations):
         uid, gid, pid = fuse_get_context()
         hashpath = self.__hashpath(path, enablepid=True)
         downloadtask = self.taskpool.query_download_task(hashpath)
-        if downloadtask:
-            # logger.debug("task already existed! pid:%d" % pid)
-            return downloadtask.wait_data(size)
-        else:
-            downloadtask = self.taskpool.download_file(hashpath, self.api, path, size)
-            return downloadtask.wait_data(size)
+        try:
+            if downloadtask:
+                return downloadtask.wait_data(offset, size)
+            else:
+                filesize = self.fileprops[path]["st_size"]
+                downloadtask = self.taskpool.download_file(hashpath, self.api, path, filesize)
+            return downloadtask.wait_data(offset, size)
+        except OpenAPIException, apiex:
+            raise FuseOSError(EIO)
 
     def link(self, linktarget, linkname):
         self.__waituploadready(linkname)
@@ -866,7 +856,7 @@ class KuaiPanFuse(LoggingMixIn, Operations):
         logger.debug("add %s to cache" % path)
         with self.rlock:
             if typecode == TYPE_FILE:
-                fileinfo = copy.copy(ROOT_ST_INFO)
+                fileinfo = copy(ROOT_ST_INFO)
                 fileinfo["st_mode"] = TYPE_FILE
                 fileinfo["st_nlink"] = 1
                 fileinfo["st_parent_dir"] = os.path.dirname(path)
@@ -875,7 +865,7 @@ class KuaiPanFuse(LoggingMixIn, Operations):
                     fileinfo["st_size"] = size
                 self.fileprops[path] = fileinfo
             else:
-                dirinfo = copy.copy(ROOT_ST_INFO)
+                dirinfo = copy(ROOT_ST_INFO)
                 dirinfo["st_parent_dir"] = os.path.dirname(path)
                 dirinfo["st_name"] = os.path.basename(path)
                 self.fileprops[path] = dirinfo
@@ -913,12 +903,19 @@ class KuaiPanFuse(LoggingMixIn, Operations):
         '''called by fuse try to write data to path'''
         hashpath = self.__hashpath(path)
         filename = os.path.basename(path)
-        writetask = self.taskpool.query_upload_task1(hashpath)
+        writetask = self.taskpool.query_upload_task(hashpath)
         if writetask:
             writetask.sendmesg("push_upload_file", (data, offset))
             self.fileprops[path]["st_size"] += len(data)
         else:
-            task = self.taskpool.upload_file(hashpath, self.api, path, filename, self.uploadqueue)
+            task = self.taskpool.upload_file(
+                        hashpath, 
+                        self.api, 
+                        path, 
+                        filename, 
+                        self.uploadqueue,
+                        handler=AfterUploadHandler(self.api, path, self)
+                    )
             task.sendmesg("start_upload_file", (data, offset))
             self.fileprops[path]["st_size"] = len(data)
         return len(data)    
@@ -936,7 +933,7 @@ class KuaiPanFuse(LoggingMixIn, Operations):
     def release(self, path, fh):
         '''called by fuse release a file'''
         hashpath = self.__hashpath(path)
-        uploadtask = self.taskpool.query_upload_task1(hashpath)
+        uploadtask = self.taskpool.query_upload_task(hashpath)
         if uploadtask:
             uploadtask.sendmesg("end_upload_file", ())
             uploadtask.sendmesg("end_task", ())
@@ -958,57 +955,69 @@ class KuaiPanFuse(LoggingMixIn, Operations):
         
     def loadtree(self):
         '''load directory tree from local disk'''
-        with self.rlock:
-            from pkg_resources import resource_filename
-            filename = resource_filename("kuaipandriver", "tree.db")
-            if os.path.exists(filename):
-                from pickle import load
-                return load(file(filename))
+        filename = gethomedir() + os.sep + "tree.db"
+        if os.path.exists(filename):
+            from pickle import load
+            return load(file(filename))
 
     def savetree(self):
         '''cache directory tree to local disk'''
-        with self.rlock:
-            from pkg_resources import resource_filename
-            filename = resource_filename("kuaipandriver", "tree.db")
-            if not self.fileprops:
-                pass
-            with open(filename, "w") as treefile:
-                from pickle import dump
-                dump(self.fileprops, treefile)
-                logger.debug("dump ok" + str(self.fileprops))
+        if not self.fileprops:
+            return
+
+        filename = gethomedir() + os.sep + "tree.db"
+        with open(filename, "w") as treefile:
+            from pickle import dump
+            dump(self.fileprops, treefile)
 
     def destroy(self, path):
         '''called by fuse destory context'''
-        logger.debug("fuse exited!")
         self.savetree()
+        SafeLRUCache.instance().save()
+        logger.debug("fuse exited!")
 
+def parse_options():
+    from optparse import OptionParser
+    parser = OptionParser()
+    parser.add_option("-c", "--configfile", dest="configfile", help="special config file fullpath")
+    parser.add_option("-f", "--foreground", dest="foreground", default=False, action="store_true", help="do not fork into background.")
+    parser.add_option("-d", "--log-level", dest="debuglevel", default="ERROR", help="log level FATAL|ERROR|WARN|DEBUG|INFO")
+    parser.add_option("-l", "--log-file-path", dest="logpath", default="/tmp/kuaipanfuse.log", help="path to log file")
+
+    (options, args) = parser.parse_args()
+
+    if not options.configfile:
+        parser.print_help()
+    else:
+        return options
+
+def parse_config(configfile):
+    if not os.path.exists(configfile):
+        print("configfile %s not existed!" % configfile)
+        sys.exit(1)
+    from json import load
+    try:
+        cfg = load(file(configfile))
+        define(mntpoint=cfg["mntpoint"], user=cfg["username"], pwd=cfg["password"], keylist=cfg["keylist"])
+    except ValueError:
+        print("config file format error")
+        sys.exit(1)
+        
 def main():
-    from kuaipandriver.common import getauthinfo, checkplatform
+    from common import checkplatform, Context
     checkplatform()
-    from kuaipandriver.kuaipanapi import KuaipanAPI
-    from requests.exceptions import RequestException
+    options = parse_options()
+    if not options:
+        return
 
-
-    islogin = False
-    while 1:
-        try:
-            mntpoint, key, secret, user, pwd = getauthinfo()
-            api = KuaipanAPI(mntpoint, key, secret, user, pwd)
-            islogin = True
-            break
-        except RequestException:
-            retry = raw_input("Your Login info already wrong, do you want to re enter it?(Y/N)")
-            if retry == "Y":
-                from kuaipandriver.common import deleteloginfo
-                deleteloginfo()
-            else:
-                break
+    parse_config(options.configfile)
+    global logger
+    logger = getlogger(logfile=options.logpath, level=options.debuglevel)
 
     try:
-        if not islogin:
-            return
+        mntpoint = Context.instance().mntpoint
         FUSE(
-                KuaiPanFuse(api), mntpoint, foreground=False, nothreads=False, 
+                KuaiPanFuse(), mntpoint, foreground=options.foreground, nothreads=False, 
                 debug=False, big_writes=True, gid=os.getgid(), uid=os.getuid(), 
                 umask='0133'
             )
